from fastapi import APIRouter, Depends, HTTPException, status
from sqlalchemy.orm import Session
from typing import Any, List

from app.db.base import get_db
from app.models.item import Item as ItemModel
from app.schemas.item import Item, ItemCreate, ItemUpdate

router = APIRouter()


@router.get("/", response_model=List[Item])
def read_items(
    skip: int = 0,
    limit: int = 100,
    db: Session = Depends(get_db),
) -> Any:
    """
    Retrieve items.
    """
    items = db.query(ItemModel).offset(skip).limit(limit).all()
    return items


@router.post("/", response_model=Item, status_code=status.HTTP_201_CREATED)
def create_item(
    *,
    item_in: ItemCreate,
    db: Session = Depends(get_db),
) -> Any:
    """
    Create new item.
    """
    db_item = ItemModel(
        name=item_in.name,
        description=item_in.description,
        is_active=item_in.is_active,
    )
    db.add(db_item)
    db.commit()
    db.refresh(db_item)
    return db_item


@router.get("/{item_id}", response_model=Item)
def read_item(
    *,
    item_id: int,
    db: Session = Depends(get_db),
) -> Any:
    """
    Get item by ID.
    """
    item = db.query(ItemModel).filter(ItemModel.id == item_id).first()
    if not item:
        raise HTTPException(
            status_code=status.HTTP_404_NOT_FOUND,
            detail="Item not found",
        )
    return item


@router.put("/{item_id}", response_model=Item)
def update_item(
    *,
    item_id: int,
    item_in: ItemUpdate,
    db: Session = Depends(get_db),
) -> Any:
    """
    Update an item.
    """
    item = db.query(ItemModel).filter(ItemModel.id == item_id).first()
    if not item:
        raise HTTPException(
            status_code=status.HTTP_404_NOT_FOUND,
            detail="Item not found",
        )
    
    update_data = item_in.model_dump(exclude_unset=True)
    for field, value in update_data.items():
        setattr(item, field, value)
    
    db.add(item)
    db.commit()
    db.refresh(item)
    return item


@router.delete("/{item_id}", status_code=status.HTTP_204_NO_CONTENT, response_model=None)
def delete_item(
    *,
    item_id: int,
    db: Session = Depends(get_db),
) -> Any:
    """
    Delete an item.
    """
    item = db.query(ItemModel).filter(ItemModel.id == item_id).first()
    if not item:
        raise HTTPException(
            status_code=status.HTTP_404_NOT_FOUND,
            detail="Item not found",
        )
    
    db.delete(item)
    db.commit()
    return None